@@ -12,7 +12,12 @@ from win32con import IDC_WAIT, MOUSEEVENTF_MOVE, MOUSEEVENTF_ABSOLUTE, MOUSEEVEN
     MOUSEEVENTF_MIDDLEDOWN, MOUSEEVENTF_MIDDLEUP, MOUSEEVENTF_RIGHTDOWN, MOUSEEVENTF_RIGHTUP, MOUSEEVENTF_WHEEL, WHEEL_DELTA
 import time
 from enum import Enum
-from typing import Any, Optional, Union, NewType
+from typing import Optional, Union, NewType
+
+__all__ = ['PlayerSettings', 'MoveMode', 'load_dictionary', 'shortcut', 'full_definition', 'Window', 'Region', 'find',
+           'move', 'click', 'left_click', 'right_click', 'double_left_click', 'triple_left_click', 'drag_and_drop',
+           'middle_drag_and_drop', 'right_drag_and_drop', 'menu_click', 'mouse_wheel', 'send_keys', 'set_combobox',
+           'set_text', 'exists', 'select_file']
 
 UI_Coordinates = NewType('UI_Coordinates', (float, float))
 UI_Element = Union[str, pywinauto.controls.uiawrapper.UIAWrapper, UI_Coordinates]
@@ -28,6 +33,7 @@ def unescape_special_char(string):
 
 class PlayerSettings:
     mouse_move_duration = 0.5
+    timeout = 10
 
 
 class MoveMode(Enum):
@@ -42,32 +48,49 @@ element_path_old = ''
 w_rOLD = None
 
 
-def load_dictionary(filename: str) -> None:
+def load_dictionary(filename_key: str, filename_def: str,encoding: str = 'utf8') -> None:
     """
     Loads a dictionary
 
-    :param filename:filename of the dictionary
+    :param filename_key: filename of the key file
+    :param filename_def: filename of the definition file
+    :param encoding: encoding of the dictionary file
     """
-    with open(filename) as fp:
-        for line in fp:
-            words = line.split("\t")
-            i = 0
-            while words[i] == '':
-                i += 1
-            variable = words[-1].translate(str.maketrans('', '', '\n\t\r'))
-            value = words[i]
-            _dictionary[variable] = value
+    abs_path = [x for x in range(99)]
+    with open(filename_key, encoding=encoding) as fp_key, open(filename_def, encoding=encoding) as fp_def:
+        for line_key, line_def in zip(fp_key, fp_def):
+            words = line_key.split("\t")
+            word = words[-1].translate(str.maketrans('', '', '\n\t\r'))
+
+            words = line_def.split("\t")
+            definition = words[-1].translate(str.maketrans('', '', '\n\t\r'))
+            level = len(words) - 1
+            #print(level)
+            abs_path[level] = definition
+            abs_definition = abs_path[0]
+            for i in range(1, level):
+                abs_definition += path_separator + abs_path[i]
+            #print(abs_definition + path_separator + definition)
+            _dictionary[word] = (abs_definition, definition)
 
 
 def shortcut(str_shortcut: str) -> str:
     """
-    Returns the element_path associated to th shortcut in the previously loaded dictionary
+    Returns the shortcut path associated to the shortcut defined in the previously loaded dictionary
 
     :param str_shortcut: shortcut
     """
-    return _dictionary[str_shortcut]
+    return _dictionary[str_shortcut][1]
 
 
+def full_definition(str_shortcut: str) -> str:
+    """
+    Returns the full element path associated to the shortcut defined in the previously loaded dictionary
+
+    :param str_shortcut: shortcut
+    """
+    return _dictionary[str_shortcut][0] + path_separator + _dictionary[str_shortcut][1]
+
 def wait_is_ready_try1(wrapper, timeout=120):
     """
     Waits until element is ready (wait while greyed, not enabled, not visible, not ready, ...) :
@@ -89,7 +112,6 @@ def wait_is_ready_try1(wrapper, timeout=120):
                     break
         except Exception:
             time.sleep(0.1)
-            pass
         if (time.time() - t0) > timeout:
             msg = "Element " + get_wrapper_path(wrapper) + "  was not found after " + str(timeout) + " s of searching."
             raise TimeoutError("Time out! ", msg)
@@ -129,7 +151,7 @@ Window = Region
 
 def find(
         element_path: Optional[UI_Element] = None,
-        timeout: Optional[float] = 120) -> UI_Element:
+        timeout: Optional[float] = None) -> UI_Element:
     """
     Finds an element
 
@@ -137,6 +159,9 @@ def find(
     :param timeout: period of time in seconds that will be allowed to find the element
     :return: Pywinauto wrapper of clicked element
     """
+    if not timeout:
+        timeout = PlayerSettings.timeout
+
     if not Region.click_desktop:
         Region.click_desktop = pywinauto.Desktop(backend='uia', allow_magic_lookup=False)
     if Region.common_path:
@@ -149,7 +174,6 @@ def find(
     entry_list = get_entry_list(element_path2)
     unique_element = None
     elements = None
-    strategy = None
     t0 = time.time()
     while (time.time() - t0) < timeout:
         while unique_element is None and not elements:
@@ -178,18 +202,17 @@ def find(
                     unique_element = candidates[int(y_x[0])][int(y_x[1])]
                 else:
                     ref_entry_list = get_entry_list(Region.common_path) + get_entry_list(y_x[0])
-                    ref_unique_element, _ = find_element(
-                        Region.click_desktop, ref_entry_list, window_candidates=[], regex_title=Region.current.regex_title)
+                    ref_unique_element, _ = find_element(Region.click_desktop, ref_entry_list,
+                                                         window_candidates=[], regex_title=Region.current.regex_title)
                     ref_r = ref_unique_element.rectangle()
                     r_y = 0
                     while r_y < nb_y:
-                        y_candidate = candidates[r_y][0].rectangle().mid_point()[1]
-                        if ref_r.top < y_candidate < ref_r.bottom:
-                            unique_element = candidates[r_y][y_x[1]]
-                            break
+                        for candidate in candidates[r_y]:
+                            y_candidate = candidate.rectangle().mid_point()[1]
+                            if ref_r.top < y_candidate < ref_r.bottom:
+                                unique_element = candidates[r_y][y_x[1]]
+                                return unique_element
                         r_y = r_y + 1
-                    else:
-                        unique_element = None
         if unique_element is not None:
             break
         time.sleep(0.1)
@@ -207,11 +230,16 @@ def move(
     Moves on element
     
     :param element_path: element path
-    :param duration: duration in seconds of the mouse move
+    :param duration: duration in seconds of the mouse move (if duration is -1 the mouse cursor doesn't move)
     :param mode: move mouse mode
     :param timeout: period of time in seconds that will be allowed to find the element
     :return: Pywinauto wrapper of clicked element
     """
+    if not duration:
+        duration = PlayerSettings.mouse_move_duration
+
+    if duration == -1:
+        return
     
     global unique_element_old
     global element_path_old
@@ -305,28 +333,43 @@ def move(
 
 def click(
         element_path: Optional[UI_Element] = None,
-        duration: Optional[float] = 0.5,
+        duration: Optional[float] = None,
         mode: Enum = MoveMode.linear,
         button: str = 'left',
-        timeout: float = 120,
+        timeout: float = None,
         wait_ready: bool = True) -> UI_Element:
     """
     Clicks on element
     
     :param element_path: element path
-    :param duration: duration in seconds of the mouse move
+    :param duration: duration in seconds of the mouse move (if duration is -1 the mouse cursor doesn't move, it just sends WM_CLICK window message, useful for minimized or non-active window).
     :param mode: move mouse mode
     :param button: mouse button: 'left','double_left', 'triple_left', 'right'
     :param timeout: period of time in seconds that will be allowed to find the element
     :param wait_ready: if True waits until the element is ready
     :return: Pywinauto wrapper of clicked element
     """
+    if not duration:
+        duration = PlayerSettings.mouse_move_duration
+
+    if not timeout:
+        timeout = PlayerSettings.timeout
+
     if element_path:
-        unique_element = move(element_path, duration=duration, mode=mode, timeout=timeout)
-        if wait_ready and isinstance(element_path, str):
-            wait_is_ready_try1(unique_element, timeout=timeout)
+        if duration == -1:
+            wrapper = find(element_path)
+            has_get_value = getattr(wrapper, "click", None)
+            if callable(has_get_value):
+                wrapper.click()
+            else:
+                wrapper.click_input()
+            return wrapper
         else:
-            unique_element = None
+            wrapper = move(element_path, duration=duration, mode=mode, timeout=timeout)
+            if wait_ready and isinstance(element_path, str):
+                wait_is_ready_try1(wrapper, timeout=timeout)
+            else:
+                wrapper = None
     if button == 'left' or button == 'double_left' or button == 'triple_left':
         win32api_mouse_event(MOUSEEVENTF_LEFTDOWN, 0, 0)
         time.sleep(.01)
@@ -347,7 +390,7 @@ def click(
         win32api_mouse_event(MOUSEEVENTF_RIGHTUP, 0, 0)
         time.sleep(.01)
     if element_path:
-        return unique_element
+        return wrapper
     else:
         return None
 
@@ -356,41 +399,36 @@ def left_click(
         element_path: UI_Element,
         duration: Optional[float] = None,
         mode: Enum = MoveMode.linear,
-        timeout: float = 120,
+        timeout: Optional[float] = None,
         wait_ready: bool = True) -> UI_Element:
     """
     Left clicks on element
     
     :param element_path: element path
-    :param duration: duration in seconds of the mouse move
+    :param duration: duration in seconds of the mouse move (if duration is -1 the mouse cursor doesn't move)
     :param mode: move mouse mode
     :param timeout: period of time in seconds that will be allowed to find the element
     :param wait_ready: if True waits until the element is ready
     :return: Pywinauto wrapper of clicked element
     """
-    if not duration:
-        duration = PlayerSettings.mouse_move_duration
     return click(element_path, duration=duration, mode=mode, button='left', timeout=timeout, wait_ready=wait_ready)
 
-
 def right_click(
         element_path: UI_Element,
         duration: Optional[float] = None,
         mode: Enum = MoveMode.linear,
-        timeout: float = 120,
+        timeout: Optional[float] = None,
         wait_ready: bool = True) -> UI_Element:
     """
     Right clicks on element
     
     :param element_path: element path
-    :param duration: duration in seconds of the mouse move
+    :param duration: duration in seconds of the mouse move (if duration is -1 the mouse cursor doesn't move)
     :param mode: move mouse mode
     :param timeout: period of time in seconds that will be allowed to find the element
     :param wait_ready: if True waits until the element is ready
     :return: Pywinauto wrapper of clicked element
     """
-    if not duration:
-        duration = PlayerSettings.mouse_move_duration
     return click(element_path, duration=duration, mode=mode, button='right', timeout=timeout, wait_ready=wait_ready)
 
 
@@ -398,21 +436,18 @@ def double_left_click(
         element_path: UI_Element,
         duration: Optional[float] = None,
         mode: Enum = MoveMode.linear,
-        timeout: float = 120,
+        timeout: Optional[float] = None,
         wait_ready: bool = True) -> UI_Element:
     """
     Double left clicks on element
     
     :param element_path: element path
-    :param duration: duration in seconds of the mouse move
+    :param duration: duration in seconds of the mouse move (if duration is -1 the mouse cursor doesn't move)
     :param mode: move mouse mode
     :param timeout: period of time in seconds that will be allowed to find the element
     :param wait_ready: if True waits until the element is ready
     :return: Pywinauto wrapper of clicked element
     """
-    
-    if not duration:
-        duration = PlayerSettings.mouse_move_duration
     return click(element_path, duration=duration, mode=mode, button='double_left', timeout=timeout, wait_ready=wait_ready)
 
 
@@ -420,20 +455,18 @@ def triple_left_click(
         element_path: UI_Element,
         duration: Optional[float] = None,
         mode: Enum = MoveMode.linear,
-        timeout: float = 120,
+        timeout: Optional[float] = None,
         wait_ready: bool = True) -> UI_Element:
     """
     Triple left clicks on element
     
     :param element_path: element path
-    :param duration: duration in seconds of the mouse move
+    :param duration: duration in seconds of the mouse move (if duration is -1 the mouse cursor doesn't move)
     :param mode: move mouse mode
     :param timeout: period of time in seconds that will be allowed to find the element
     :param wait_ready: if True waits until the element is ready
     :return: Pywinauto wrapper of clicked element
     """
-    if not duration:
-        duration = PlayerSettings.mouse_move_duration
     return click(element_path, duration=duration, mode=mode, button='triple_left', timeout=timeout, wait_ready=wait_ready)
 
 
@@ -442,20 +475,17 @@ def drag_and_drop(
         element_path2: UI_Element,
         duration: Optional[float] = None,
         mode: Enum = MoveMode.linear,
-        timeout: float = 120) -> UI_Element:
+        timeout: Optional[float] = None) -> UI_Element:
     """
     Drags and drop with left button pressed from element_path1 to element_path2.
     
     :param element_path1: element path
     :param element_path2: element path
-    :param duration: duration in seconds of the mouse move
+    :param duration: duration in seconds of the mouse move (if duration is -1 the mouse cursor doesn't move)
     :param mode: move mouse mode
     :param timeout: period of time in seconds that will be allowed to find the element
     :return: Pywinauto wrapper with element_path2
     """
-    
-    if not duration:
-        duration = PlayerSettings.mouse_move_duration
     move(element_path1, duration=duration, mode=mode, timeout=timeout)
     win32api_mouse_event(MOUSEEVENTF_LEFTDOWN, 0, 0)
     unique_element = move(element_path2, duration=duration, timeout=timeout)
@@ -468,19 +498,17 @@ def middle_drag_and_drop(
         element_path2: UI_Element,
         duration: Optional[float] = None,
         mode: Enum = MoveMode.linear,
-        timeout: float = 120) -> UI_Element:
+        timeout: Optional[float] = None) -> UI_Element:
     """
     Drags and drop with middle button pressed from element_path1 to element_path2.
     
     :param element_path1: element path
     :param element_path2: element path
-    :param duration: duration in seconds of the mouse move
+    :param duration: duration in seconds of the mouse move (if duration is -1 the mouse cursor doesn't move)
     :param mode: move mouse mode
     :param timeout: period of time in seconds that will be allowed to find the element
     :return: Pywinauto wrapper with element_path2
     """
-    if not duration:
-        duration = PlayerSettings.mouse_move_duration
     move(element_path1, duration=duration, mode=mode, timeout=timeout)
     win32api_mouse_event(MOUSEEVENTF_MIDDLEDOWN, 0, 0)
     unique_element = move(element_path2, duration=duration, mode=mode, timeout=timeout)
@@ -493,19 +521,17 @@ def right_drag_and_drop(
         element_path2: UI_Element,
         duration: Optional[float] = None,
         mode: Enum = MoveMode.linear,
-        timeout: float = 120) -> UI_Element:
+        timeout: Optional[float] = None) -> UI_Element:
     """
     Drags and drop with right button pressed from element_path1 to element_path2.
     
     :param element_path1: element path
     :param element_path2: element path
-    :param duration: duration in seconds of the mouse move
+    :param duration: duration in seconds of the mouse move (if duration is -1 the mouse cursor doesn't move)
     :param mode: move mouse mode
     :param timeout: period of time in seconds that will be allowed to find the element
     :return: Pywinauto wrapper with element_path2
     """
-    if not duration:
-        duration = PlayerSettings.mouse_move_duration
     move(element_path1, duration=duration, mode=mode, timeout=timeout)
     win32api_mouse_event(MOUSEEVENTF_RIGHTDOWN, 0, 0)
     unique_element = move(element_path2, duration=duration, mode=mode, timeout=timeout)
@@ -519,20 +545,18 @@ def menu_click(
         duration: Optional[float] = None,
         mode: Enum = MoveMode.linear,
         menu_type: str = 'QT',
-        timeout: float = 120) -> UI_Element:
+        timeout: Optional[float] = None) -> UI_Element:
     """
     Clicks on menu item.
     
     :param element_path: element path
     :param menu_path: menu path
-    :param duration: duration in seconds of the mouse move
+    :param duration: duration in seconds of the mouse move (if duration is -1 the mouse cursor doesn't move)
     :param mode: move mouse mode
     :param menu_type: menu type ('QT', 'NPP')
     :param timeout: period of time in seconds that will be allowed to find the element
     :return: Pywinauto wrapper of the clicked item
     """
-    if not duration:
-        duration = PlayerSettings.mouse_move_duration
     menu_entry_list = menu_path.split(path_separator)
     if menu_type == 'QT':
         menu_entry_list = [''] + menu_entry_list
@@ -603,7 +627,7 @@ def send_keys(
     """
     for r in (('(', '{(}'),  (')', '{)}'), ('+', '{+}')):
         str_keys = str_keys.replace(*r)
-    pywinauto.keyboard.send_keys(
+    pywinauto.keyboard.send_keys(   # lgtm [py/call/wrong-named-argument]
         str_keys,
         pause=pause,
         with_spaces=with_spaces,
@@ -619,18 +643,16 @@ def set_combobox(
         value: str,
         duration: Optional[float] = None,
         mode: Enum = MoveMode.linear,
-        timeout: float = 120) -> None:
+        timeout: Optional[float] = None) -> None:
     """
     Sets the value of a combobox.
     
     :param element_path: element path
     :param value: value of the combobox
-    :param duration: duration in seconds of the mouse move
+    :param duration: duration in seconds of the mouse move (if duration is -1 the mouse cursor doesn't move)
     :param mode: move mouse mode
     :param timeout: period of time in seconds that will be allowed to find the element
     """
-    if not duration:
-        duration = PlayerSettings.mouse_move_duration
     left_click(element_path, duration=duration, mode=mode, timeout=timeout)
     time.sleep(0.9)
     send_keys(value + "{ENTER}")
@@ -641,20 +663,18 @@ def set_text(
         value: str,
         duration: Optional[float] = None,
         mode: Enum = MoveMode.linear,
-        timeout: float = 120,
+        timeout: Optional[float] = None,
         pause: float = 0.1) -> None:
     """
     Sets the value of a text field.
     
     :param element_path: element path
     :param value: value of the combobox
-    :param duration: duration in seconds of the mouse move
+    :param duration: duration in seconds of the mouse move (if duration is -1 the mouse cursor doesn't move)
     :param mode: move mouse mode
     :param timeout: period of time in seconds that will be allowed to find the element
     :param pause: pause in seconds between each typed key
     """
-    if not duration:
-        duration = PlayerSettings.mouse_move_duration
     double_left_click(element_path, duration=duration, mode=mode, timeout=timeout)
     send_keys("{VK_CONTROL down}a{VK_CONTROL up}", pause=0)
     time.sleep(0.1)
@@ -663,7 +683,7 @@ def set_text(
 
 def exists(
         element_path: UI_Element,
-        timeout: float = 120) -> UI_Element:
+        timeout: Optional[float] = None) -> UI_Element:
     """
     Tests if en UI_Element exists.
     
