@@ -13,10 +13,11 @@ import keyboard
 import mouse
 from collections import namedtuple
 import pyperclip
-import math
 import codecs
 from .core import *
 
+__all__ = ['Recorder', 'overlay_add_mode_icon', 'overlay_add_progress_icon']
+
 ElementEvent = namedtuple('ElementEvent', ['strategy', 'rectangle', 'path'])
 SendKeysEvent = namedtuple('SendKeysEvent', ['line'])
 MouseWheelEvent = namedtuple('MouseWheelEvent', ['delta'])
@@ -66,9 +67,6 @@ def write_in_file(events, relative_coordinate_mode=False, menu_click=False):
 	record_file_name = home_dir / Path('recorded ' + time.asctime().replace(':', '_') + '.py')
 	print('Recording in file: ' + str(record_file_name.absolute()))
 	script = "# encoding: {}\n\n".format(sys.getdefaultencoding())
-	script += u"import os, sys\n"
-	script += u"script_dir = os.path.dirname(__file__)\n"
-	script += u"sys.path.append(script_dir)\n"
 	script += "from pywinauto_recorder.player import *\n\n"
 	common_path = ''
 	common_window = ''
@@ -519,7 +517,12 @@ def get_send_keys_strings(keyboard_events):
 		return get_typed_keys(keyboard_events)
 
 
+t0_progress_icon_timings = time.time()
+progress_icon_timings = [0, 0, 0, 0, 0, 0, 0]
+
+
 def overlay_add_progress_icon(main_overlay, i, x, y):
+	global t0_progress_icon_timings
 	main_overlay.add(
 		geometry=oaam.Shape.rectangle, x=x, y=y, width=52, height=52,
 		color=(0, 0, 0), thickness=1, brush=oaam.Brush.solid, brush_color=(255, 255, 254))
@@ -527,10 +530,19 @@ def overlay_add_progress_icon(main_overlay, i, x, y):
 		geometry=oaam.Shape.triangle,
 		xyrgb_array=((x + 1, y + 1, 255, 255, 254), (x + 1, y + 52, 128, 128, 128), (x + 51, y + 52, 255, 255, 254)),
 		thickness=0)
+	dt = time.time() - t0_progress_icon_timings
+	nb_dt = int(dt/0.01)
+	if nb_dt > 255:
+		nb_dt = int(255)
+	progress_icon_timings[i % 6 -1] = nb_dt
+	
 	for b in range(i % 6):
+		c = progress_icon_timings[b]
 		main_overlay.add(
 			geometry=oaam.Shape.rectangle, x=x + 6, y=y + 6 + b * 8, width=40, height=6,
-			color=(0, 255, 0), thickness=1, brush=oaam.Brush.solid, brush_color=(0, 200, 0))
+			# color=(0, 255, 0), thickness=1, brush=oaam.Brush.solid, brush_color=(0, 200, 0))
+			color=(c, int(255 - c/2), c ), thickness=1, brush=oaam.Brush.solid, brush_color=(c, int(255 - c), 0))
+	t0_progress_icon_timings = time.time()
 
 
 def overlay_add_mode_icon(main_overlay, hicon, x, y):
@@ -557,6 +569,7 @@ class Recorder(Thread):
 		self.desktop = pywinauto.Desktop(backend='uia', allow_magic_lookup=False)
 		self.daemon = True
 		self.event_list = []
+		self._copy_count = 0
 		self._mode = "Info"
 		self._process_menu_click_mode = True
 		self._smart_mode = False
@@ -613,8 +626,7 @@ class Recorder(Thread):
 								thickness=1, color=(255, 0, 0), brush=oaam.Brush.solid,
 								brush_color=(255, 200, 0))
 							return '#[' + wrapper_path2 + ',' + str(r_x) + ']'
-					else:
-						return None
+					return None
 		return None
 
 	def __find_unique_element_array_2d(self, wrapper_rectangle, elements):
@@ -664,15 +676,11 @@ class Recorder(Thread):
 				and keyboard.key_to_scan_codes("shift")[0] in keyboard._pressed_events
 				and keyboard.key_to_scan_codes("ctrl")[0] in keyboard._pressed_events):
 			if self.last_element_event:
+				self._copy_count = 2
 				x, y = win32api.GetCursorPos()
 				l_e_e = self.last_element_event
 				dx, dy = compute_dx_dy(x, y, l_e_e.rectangle)
 				str_dx, str_dy = "{:.2f}".format(round(dx * 100, 2)), "{:.2f}".format(round(dy * 100, 2))
-				#self.info_overlay.add(
-				#	geometry=oaam.Shape.image, hicon=IconSet.hicon_clipboard, x=x, y=l_e_e.rectangle.top - 70)
-				info_left = (dx+1)/2 * (self.screen_width - 500)
-				info_top = (dy+1)/2 * (self.screen_height - 50)
-				overlay_add_mode_icon(self.info_overlay, IconSet.hicon_clipboard, info_left, info_top)
 				i = l_e_e.path.find(path_separator)
 				window_title = l_e_e.path[0:i]
 				# element_path = l_e_e.path[i+len(path_separator):]
@@ -699,7 +707,6 @@ class Recorder(Thread):
 	def __display_info_tip(self, x, y, wrapper):
 		tooltip_width = 500
 		tooltip_height = 25
-		r = wrapper.rectangle()
 		
 		if wrapper != self.wrapper_old_info_tip:
 			if self.wrapper_old_info_tip:
@@ -736,10 +743,7 @@ class Recorder(Thread):
 				text2 = text2 + '\n'
 				text_width = 0
 				tooltip2_height = tooltip2_height + 16
-		dx, dy = (x - r.left) / r.width(), (y - r.top) / r.height()
-		info_left = dx * (self.screen_width - tooltip_width)
-		info_top = dy * (self.screen_height - (tooltip_height+tooltip2_height))
-		if x>self.screen_width /2:
+		if x > self.screen_width / 2:
 			info_left = 9
 		else:
 			info_left = self.screen_width - tooltip_width - 10
@@ -769,72 +773,74 @@ class Recorder(Thread):
 			font_size=16, text_color=(0, 0, 0), brush=oaam.Brush.solid, brush_color=(180, 254, 255),
 			geometry=oaam.Shape.rectangle, thickness=0
 		)
-		if True:
-			text = ""
-			try:
-				has_get_value = getattr(wrapper, "get_value", None)
-				if callable(has_get_value):
-					text = "wrapper.get_value(): " + wrapper.get_value()
-			except:
-				has_get_value = False
-				pass
+
+		text = ""
+		try:
+			has_get_value = getattr(wrapper, "get_value", None)
+			if callable(has_get_value):
+				text = "wrapper.get_value(): " + wrapper.get_value()
+		except:
+			has_get_value = False
+		has_legacy_properties = getattr(wrapper, "legacy_properties", None)
+		if callable(has_legacy_properties):
 			if wrapper.legacy_properties()['Value']:
 				if not has_get_value or (has_get_value and wrapper.get_value() != wrapper.legacy_properties()['Value']):
 					if text:
 						text = text + "\n"
 					text = text + " wrapper.legacy_properties()['Value']: " + wrapper.legacy_properties()['Value']
-			str_name, str_type, _, _ = get_entry(end_path.split(path_separator)[-1])
-			try:
-				if str_type in ["Button", "CheckBox", "RadioButton", "GroupBox"]:
-					if text:
+		str_name, str_type, _, _ = get_entry(end_path.split(path_separator)[-1])
+		try:
+			if str_type in ["Button", "CheckBox", "RadioButton", "GroupBox"]:
+				if text:
+					text = text + "\n"
+				text = text + "wrapper.legacy_properties()['State']: " + str(wrapper.legacy_properties()['State']) + "\n"
+				# l'un ou l'autre fonctionne mais pas les 2 en même temps! Pourquoi? Une exception??
+				#text = text + "wrapper.get_toggle_state(): " + str(wrapper.get_toggle_state()) + "\n"
+
+
+				from pywinauto.controls.win32_controls import ButtonWrapper
+				text = text + "ButtonWrapper(wrapper).is_checked(): " + str(ButtonWrapper(wrapper).is_checked()) + "\n"
+
+			elif str_type in ["ComboBox"]:
+				from pywinauto.controls.win32_controls import ComboBoxWrapper
+				if text:
+					text = text + "\n"
+					text = text + " ComboBoxWrapper(wrapper).selected_text(): " + str(ComboBoxWrapper(wrapper).selected_text()) + "\n"
+			elif str_type in ["Edit"]:
+				if text:
+					str_wrapper_text_block = str(wrapper.text_block())
+					if str_wrapper_text_block != str_name:
 						text = text + "\n"
-					text = text + "wrapper.legacy_properties()['State']: " + str(wrapper.legacy_properties()['State']) + "\n"
-					# l'un ou l'autre fonctionne mais pas les 2 en même temps! Pourquoi?
-					#text = text + "wrapper.get_toggle_state(): " + str(wrapper.get_toggle_state()) + "\n"
-					
-					
-					from pywinauto.controls.win32_controls import ButtonWrapper
-					text = text + "ButtonWrapper(wrapper).is_checked(): " + str(ButtonWrapper(wrapper).is_checked()) + "\n"
-					
-				elif str_type in ["ComboBox"]:
-					from pywinauto.controls.win32_controls import ComboBoxWrapper
-					if text:
-						text = text + "\n"
-						text = text + " ComboBoxWrapper(wrapper).selected_text(): " + str(ComboBoxWrapper(wrapper).selected_text()) + "\n"
-				elif str_type in ["Edit"]:
-					if text:
-						str_wrapper_text_block = str(wrapper.text_block())
-						if str_wrapper_text_block != str_name:
-							text = text + "\n"
-							text = text + "wrapper.text_block(): " + str_wrapper_text_block + "\n"
-			except:
-				pass
-			tooltip3_height = tooltip_height
-			text3 = ''
-			text_width = 0
-			for c in text:
-				if c == '\n':
-					text_width = 0
-					tooltip3_height = tooltip3_height + 16
-				else:
-					text_width = text_width + 6.8
-				text3 = text3 + c
-				if text_width > tooltip_width:
-					text3 = text3 + '\n'
-					text_width = 0
-					tooltip3_height = tooltip3_height + 16
-			if text3:
-				self.info_overlay.add(
-					geometry=oaam.Shape.rectangle, x=info_left, y=info_top - 4 + tooltip_height + tooltip2_height, width=tooltip_width,
-					height=tooltip3_height, thickness=1, color=(0, 0, 0), brush=oaam.Brush.solid, brush_color=(2, 254, 255))
-				self.info_overlay.add(
-					x=info_left + 5, y=info_top + tooltip_height + tooltip2_height, width=tooltip_width-7,
-					height=tooltip3_height-5,
-					text=text3,
-					text_format="win32con.DT_LEFT|win32con.DT_TOP|win32con.DT_WORDBREAK|win32con.DT_NOCLIP|win32con.DT_VCENTER",
-					font_size=16, text_color=(0, 0, 0), brush=oaam.Brush.solid, brush_color=(2, 254, 255),
-					geometry=oaam.Shape.rectangle, thickness=0
-				)
+						text = text + "wrapper.text_block(): " + str_wrapper_text_block + "\n"
+
+		except:
+			pass
+		tooltip3_height = tooltip_height
+		text3 = ''
+		text_width = 0
+		for c in text:
+			if c == '\n':
+				text_width = 0
+				tooltip3_height = tooltip3_height + 16
+			else:
+				text_width = text_width + 6.8
+			text3 = text3 + c
+			if text_width > tooltip_width:
+				text3 = text3 + '\n'
+				text_width = 0
+				tooltip3_height = tooltip3_height + 16
+		if text3:
+			self.info_overlay.add(
+				geometry=oaam.Shape.rectangle, x=info_left, y=info_top - 4 + tooltip_height + tooltip2_height, width=tooltip_width,
+				height=tooltip3_height, thickness=1, color=(0, 0, 0), brush=oaam.Brush.solid, brush_color=(2, 254, 255))
+			self.info_overlay.add(
+				x=info_left + 5, y=info_top + tooltip_height + tooltip2_height, width=tooltip_width-7,
+				height=tooltip3_height-5,
+				text=text3,
+				text_format="win32con.DT_LEFT|win32con.DT_TOP|win32con.DT_WORDBREAK|win32con.DT_NOCLIP|win32con.DT_VCENTER",
+				font_size=16, text_color=(0, 0, 0), brush=oaam.Brush.solid, brush_color=(2, 254, 255),
+				geometry=oaam.Shape.rectangle, thickness=0
+			)
 		loop_duration = time.time() - self._loop_t0
 		while loop_duration < 0.1:
 			time.sleep(0.01)
@@ -866,11 +872,12 @@ class Recorder(Thread):
 		strategies = [Strategy.unique_path, Strategy.array_2D, Strategy.array_1D]
 		i_strategy = 0
 		while self.mode != "Quit":
+			i = i + 1
 			try:
 				self._loop_t0 = time.time()
 				self.main_overlay.clear_all()
-				x, y = win32api.GetCursorPos()
-				wrapper = self.desktop.from_point(x, y)
+				cursor_pos = win32api.GetCursorPos()
+				wrapper = self.desktop.from_point(*cursor_pos)
 				if wrapper is None:
 					continue
 				wrapper_path = get_wrapper_path(wrapper)
@@ -890,10 +897,18 @@ class Recorder(Thread):
 					unique_candidate, elements = find_element(self.desktop, entry_list, window_candidates=[])
 				strategy = strategies[i_strategy]
 				unique_wrapper_path = None
+				# *** ----> this block of code must start a new while iteration if mouse cursor is outside wrapper rectangle
+				# => add tests to leave if mouse cursor is outside wrapper rectangle
+				wrapper_rectangle = wrapper.rectangle()
 				if strategy == Strategy.unique_path:
+					x_new, y_new = win32api.GetCursorPos()
+					if not ((wrapper_rectangle.left < x_new < wrapper_rectangle.right) and (
+							wrapper_rectangle.top < y_new < wrapper_rectangle.bottom)):
+						i_strategy = 0
+						continue
 					if unique_candidate is not None:
 						unique_wrapper_path = get_wrapper_path(unique_candidate)
-						r = wrapper.rectangle()
+						r = wrapper_rectangle
 						self.main_overlay.add(
 							geometry=oaam.Shape.rectangle, x=r.left, y=r.top, width=r.width(), height=r.height(),
 							thickness=1, color=(0, 128, 0), brush=oaam.Brush.solid, brush_color=(0, 255, 0))
@@ -904,15 +919,26 @@ class Recorder(Thread):
 								geometry=oaam.Shape.rectangle, x=r.left, y=r.top, width=r.width(), height=r.height(),
 								thickness=1, color=(0, 128, 0), brush=oaam.Brush.solid, brush_color=(255, 0, 0))
 				if strategy == Strategy.array_1D and elements:
+					x_new, y_new = win32api.GetCursorPos()
+					if not ((wrapper_rectangle.left < x_new < wrapper_rectangle.right) and (
+							wrapper_rectangle.top < y_new < wrapper_rectangle.bottom)):
+						i_strategy = 0
+						continue
 					unique_array_1d = self.__find_unique_element_array_1d(wrapper.rectangle(), elements)
 					if unique_array_1d is not None:
 						unique_wrapper_path = wrapper_path + unique_array_1d
 					else:
 						strategy = Strategy.array_2D
 				if strategy == Strategy.array_2D and elements:
+					x_new, y_new = win32api.GetCursorPos()
+					if not ((wrapper_rectangle.left < x_new < wrapper_rectangle.right) and (
+							wrapper_rectangle.top < y_new < wrapper_rectangle.bottom)):
+						i_strategy = 0
+						continue
 					unique_array_2d = self.__find_unique_element_array_2d(wrapper.rectangle(), elements)
 					if unique_array_2d is not None:
 						unique_wrapper_path = wrapper_path + unique_array_2d
+				# <----- ***
 				if unique_wrapper_path is not None:
 					self.last_element_event = ElementEvent(strategy, wrapper.rectangle(), unique_wrapper_path)
 					if self.event_list and unique_wrapper_path is not None:
@@ -920,8 +946,7 @@ class Recorder(Thread):
 				nb_icons = 0
 				if self.mode == "Record":
 					overlay_add_mode_icon(self.main_overlay, IconSet.hicon_record, 10, 10)
-					#overlay_add_record_icon(self.main_overlay, 10, 10)
-					nb_icons = nb_icons + 1
+					nb_icons += 1
 				elif self.mode == "Play":
 					while self.mode == "Play":
 						self.info_overlay.clear_all()
@@ -930,8 +955,6 @@ class Recorder(Thread):
 						self.info_overlay.refresh()
 						self.main_overlay.refresh()
 						time.sleep(1.0)
-				#elif self.mode == "Info":
-				#	overlay_add_pause_icon(self.main_overlay, 10, 10)
 				elif self.mode == "Stop":
 					while self.mode == "Stop":
 						self.info_overlay.clear_all()
@@ -943,16 +966,21 @@ class Recorder(Thread):
 						time.sleep(1.0)
 				if self.mode in ["Record", "Info"]:
 					overlay_add_progress_icon(self.main_overlay, i, 10+60*nb_icons, 10)
-					nb_icons = nb_icons + 1
+					nb_icons += 1
+				if self.mode == "Info":
 					overlay_add_mode_icon(self.main_overlay, IconSet.hicon_search, 10 + 60 * nb_icons, 10)
-					nb_icons = nb_icons + 1
+					nb_icons += 1
 				if self.smart_mode:
 					overlay_add_mode_icon(self.main_overlay, IconSet.hicon_light_on, 10 + 60 * nb_icons, 10)
-					nb_icons = nb_icons + 1
-				i = i + 1
+					nb_icons += 1
+				if self._copy_count > 0:
+					overlay_add_mode_icon(self.main_overlay, IconSet.hicon_clipboard, 10 + 60 * nb_icons, 10)
+					nb_icons += 1
+					self._copy_count = self._copy_count - 1
+				
 				self.main_overlay.refresh()
 				if self.mode == "Info":
-					self.__display_info_tip(x, y, wrapper)
+					self.__display_info_tip(*cursor_pos, wrapper)
 				else:
 					time.sleep(0.005)  # main_overlay.clear_all() doit attendre la fin de main_overlay.refresh()
 			except Exception as e:
@@ -964,7 +992,8 @@ class Recorder(Thread):
 			self.stop_recording()
 		mouse.unhook_all()
 		keyboard.unhook_all()
-		self.quit()
+		self.main_overlay.quit()
+		self.info_overlay.quit()
 		print("Run end")
 
 	@property
@@ -1037,4 +1066,5 @@ class Recorder(Thread):
 		self.main_overlay.refresh()
 		self.info_overlay.refresh()
 		self.mode = 'Quit'
+		self.join()
 		print("Quit")
